"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Receiver = void 0;
const common_1 = require("@skyway-sdk/common");
const log = new common_1.Logger('packages/sfu-client/src/connection/receiver.ts');
class Receiver {
    constructor(subscription, _api, _transportRepository, _localPersonId, _bot, _iceManager) {
        this.subscription = subscription;
        this._api = _api;
        this._transportRepository = _transportRepository;
        this._localPersonId = _localPersonId;
        this._bot = _bot;
        this._iceManager = _iceManager;
    }
    consume() {
        return __awaiter(this, void 0, void 0, function* () {
            let rtpCapabilities = this._transportRepository.rtpCapabilities;
            if (!rtpCapabilities) {
                log.debug('[start] getCapabilities');
                rtpCapabilities = yield this._api.getRtpCapabilities({
                    botId: this._bot.id,
                    forwardingId: this.subscription.publication.id,
                });
                log.debug('[end] getCapabilities');
                yield this._transportRepository.loadDevice(rtpCapabilities);
            }
            log.debug('[start] createConsumer', { subscription: this.subscription });
            const { consumerOptions, transportOptions, transportId, producerId } = yield this._api.createConsumer({
                botId: this._bot.id,
                forwardingId: this.subscription.publication.id,
                rtpCapabilities,
                subscriptionId: this.subscription.id,
                subscriberId: this.subscription.subscriber.id,
            });
            if (transportOptions) {
                this._transportRepository.createTransport(this._localPersonId, transportOptions, 'recv', this._iceManager.iceServers);
            }
            this._transport = this._transportRepository.getTransport(this._localPersonId, transportId);
            if (!this._transport) {
                throw new common_1.SkyWayError({
                    type: 'notFound',
                    message: 'receiver sfuTransport not found',
                    payload: { transportOptions },
                    path: log.prefix,
                });
            }
            log.debug('[end] createConsumer');
            const consumer = yield this._transport.msTransport.consume(Object.assign(Object.assign({}, consumerOptions), { producerId }));
            this._consumer = consumer;
            return { track: consumer.track, codecs: consumer.rtpParameters.codecs };
        });
    }
    unconsume() {
        if (!this._consumer) {
            log.debug('unconsume failed, consumer not exist', {
                subscription: this.subscription,
            });
            return;
        }
        this._consumer.close();
        this._consumer = undefined;
    }
    getStats() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            return (_b = (_a = this._transport) === null || _a === void 0 ? void 0 : _a.msTransport.getStats()) !== null && _b !== void 0 ? _b : {};
        });
    }
    get pc() {
        var _a;
        return (_a = this._transport) === null || _a === void 0 ? void 0 : _a.pc;
    }
}
exports.Receiver = Receiver;
//# sourceMappingURL=receiver.js.map