"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Sender = void 0;
const common_1 = require("@skyway-sdk/common");
const core_1 = require("@skyway-sdk/core");
const log = new common_1.Logger('packages/sfu-client/src/connection/sender.ts');
class Sender {
    constructor(publication, _api, _transportRepository, _localPersonId, _bot, _iceManager) {
        this.publication = publication;
        this._api = _api;
        this._transportRepository = _transportRepository;
        this._localPersonId = _localPersonId;
        this._bot = _bot;
        this._iceManager = _iceManager;
        this._disposer = [];
    }
    startForwarding(configure) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.publication.contentType === 'data') {
                throw new common_1.SkyWayError({
                    type: 'invalidParameter',
                    message: 'sfu not support dataStream',
                    path: log.prefix,
                });
            }
            log.debug('[start] Sender startForwarding', {
                botId: this._bot.id,
                publicationId: this.publication.id,
                contentType: this.publication.contentType,
                maxSubscribers: configure.maxSubscribers,
            });
            const { rtpCapabilities, transportOptions, forwardingId, transportId } = yield this._api.startForwarding({
                botId: this._bot.id,
                publicationId: this.publication.id,
                contentType: this.publication.contentType,
                maxSubscribers: configure.maxSubscribers,
                publisherId: this.publication.publisher.id,
            });
            this.forwardingId = forwardingId;
            if (rtpCapabilities) {
                yield this._transportRepository.loadDevice(rtpCapabilities);
            }
            if (transportOptions) {
                this._transport = this._transportRepository.createTransport(this._localPersonId, transportOptions, 'send', this._iceManager.iceServers);
                this._transport.msTransport.on('produce', (producerOptions, callback, errback) => Sender._onProduce(this._transport.id, this._api, this._bot.id)(producerOptions, callback, errback));
            }
            this._transport = this._transportRepository.getTransport(this._localPersonId, transportId);
            if (!this._transport) {
                throw new common_1.SkyWayError({
                    type: 'notFound',
                    message: 'sender sfuTransport not found',
                    payload: { transportOptions },
                    path: log.prefix,
                });
            }
            yield this._produce(this._transport.msTransport);
            log.debug('[end] Sender startForwarding', {
                forwardingId,
            });
            return forwardingId;
        });
    }
    _produce(transport) {
        var _b, _c, _d, _e, _f, _g;
        return __awaiter(this, void 0, void 0, function* () {
            const stream = this.publication.stream;
            if (!stream) {
                throw new common_1.SkyWayError({
                    type: 'notFound',
                    message: 'publication not have stream',
                    path: log.prefix,
                });
            }
            this._disposer.push(stream._onReplaceTrack.add((track) => __awaiter(this, void 0, void 0, function* () {
                yield this._replaceTrack(track);
            })).removeListener);
            const producerOptions = {
                track: stream.track,
                appData: { forwardingId: this.forwardingId },
            };
            const encodings = this.publication.encodings;
            if (encodings) {
                producerOptions.encodings = encodings;
            }
            this.publication._onEncodingsChanged.add((encodings) => __awaiter(this, void 0, void 0, function* () {
                yield (0, core_1.setEncodingParams)(producer.rtpSender, encodings).catch((e) => {
                    log.error('_onEncodingsChanged failed', e);
                });
            }));
            const codecCapabilities = this.publication.codecCapabilities;
            const deviceCodecs = (_c = (_b = this._transportRepository.rtpCapabilities) === null || _b === void 0 ? void 0 : _b.codecs) !== null && _c !== void 0 ? _c : [];
            const codecMimeType = (_e = (_d = codecCapabilities.find((codec) => deviceCodecs
                .map((rtp) => rtp.mimeType.toLowerCase())
                .includes(codec.mimeType.toLowerCase()))) === null || _d === void 0 ? void 0 : _d.mimeType) !== null && _e !== void 0 ? _e : '';
            const codec = deviceCodecs.find((c) => c.mimeType.toLowerCase() === codecMimeType.toLocaleLowerCase());
            if (codec) {
                producerOptions.codec = codec;
            }
            // apply opusDtx
            const opusDtx = (_g = (_f = codecCapabilities.find((c) => c.mimeType.toLowerCase() === 'audio/opus')) === null || _f === void 0 ? void 0 : _f.parameters) === null || _g === void 0 ? void 0 : _g.usedtx;
            if (opusDtx !== false) {
                producerOptions.codecOptions = Object.assign(Object.assign({}, producerOptions.codecOptions), { opusDtx: true });
            }
            const producer = yield transport.produce(producerOptions);
            this._producer = producer;
            if ((0, core_1.isSafari)()) {
                if (encodings.length > 0) {
                    // answerをsRDした後に設定する
                    yield (0, core_1.setEncodingParams)(producer.rtpSender, encodings).catch((e) => {
                        log.error('_onEncodingsChanged failed', e);
                    });
                }
            }
        });
    }
    unproduce() {
        if (!this._producer) {
            throw new common_1.SkyWayError({
                type: 'notFound',
                message: 'producer not exist',
                path: log.prefix,
            });
        }
        this._producer.close();
        this._producer = undefined;
    }
    _replaceTrack(track) {
        var _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            yield ((_c = (_b = this._producer) === null || _b === void 0 ? void 0 : _b.replaceTrack) === null || _c === void 0 ? void 0 : _c.call(_b, { track }));
        });
    }
    getStats() {
        var _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            return (_c = (_b = this._transport) === null || _b === void 0 ? void 0 : _b.msTransport.getStats()) !== null && _c !== void 0 ? _c : {};
        });
    }
    close() {
        this._disposer.forEach((d) => d());
        this._disposer = [];
    }
    get pc() {
        var _b;
        return (_b = this._transport) === null || _b === void 0 ? void 0 : _b.pc;
    }
}
exports.Sender = Sender;
_a = Sender;
Sender._onProduce = (transportId, api, botId) => (producerOptions, callback, errback) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        log.debug('[start] createProducer', { producerOptions });
        const id = yield api.createProducer({
            botId,
            forwardingId: producerOptions.appData.forwardingId,
            transportId,
            producerOptions,
        });
        log.debug('[end] createProducer', { producerOptions });
        callback({ id });
    }
    catch (error) {
        log.error('[failed] createProducer', {
            error,
            producerOptions,
        });
        errback(error);
    }
});
//# sourceMappingURL=sender.js.map