"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SfuTransport = void 0;
const core_1 = require("@skyway-sdk/core");
const log = new core_1.Logger('packages/sfu-client/src/connection/transport/transport.ts');
class SfuTransport {
    constructor(msTransport, _api) {
        this.msTransport = msTransport;
        this._api = _api;
        this._onConnectionStateChange = (state) => __awaiter(this, void 0, void 0, function* () {
            switch (state) {
                case 'connected':
                    break;
                case 'failed':
                    break;
                case 'disconnected':
                    // todo
                    break;
            }
        });
        this._onConnect = (transportId) => ({ dtlsParameters, }, callback, errback) => __awaiter(this, void 0, void 0, function* () {
            try {
                log.debug('[start] Sender connect', { transportId });
                yield this._api.connect({ transportId, dtlsParameters });
                log.debug('[end] Sender connect', { transportId });
                callback();
            }
            catch (error) {
                log.error('Sender connect failed', {
                    error,
                    transportId,
                });
                errback(error);
            }
        });
        msTransport.on('connect', (...args) => this._onConnect(msTransport.id)(args[0], args[1], args[2]));
        msTransport.on('connectionstatechange', this._onConnectionStateChange);
    }
    get pc() {
        var _a, _b;
        return (_b = (_a = this.msTransport) === null || _a === void 0 ? void 0 : _a._handler) === null || _b === void 0 ? void 0 : _b._pc;
    }
    get id() {
        return this.msTransport.id;
    }
    close() {
        var _a;
        this.msTransport.removeListener('connectionstatechange', this._onConnectionStateChange);
        // suppress firefox [RTCPeerConnection is gone] Exception
        if ((_a = this.pc) === null || _a === void 0 ? void 0 : _a.peerIdentity) {
            this.pc.peerIdentity.catch(() => { });
        }
        this.msTransport.close();
    }
}
exports.SfuTransport = SfuTransport;
//# sourceMappingURL=transport.js.map