import { Event } from '@skyway-sdk/common';
import { LocalAudioStream, LocalPersonImpl, LocalVideoStream, Member, MemberImpl, MemberType, Publication, RemoteMemberImplInterface, SkyWayChannelImpl, SkyWayContext } from '@skyway-sdk/core';
import { SfuRestApiClient } from '@skyway-sdk/sfu-api-client';
import { SfuClientPlugin } from '.';
import { SFUConnection } from './connection';
import { TransportRepository } from './connection/transport/transportRepository';
import { Forwarding, ForwardingConfigure } from './forwarding';
export declare class SfuBotMember extends MemberImpl implements RemoteMemberImplInterface {
    readonly side = "remote";
    static readonly subtype = "sfu";
    readonly subtype = "sfu";
    readonly type: MemberType;
    private readonly _context;
    private readonly _transportRepository;
    private _connections;
    readonly onForwardingStarted: Event<{
        forwarding: Forwarding;
    }>;
    readonly onForwardingStopped: Event<{
        forwarding: Forwarding;
    }>;
    readonly onForwardingStateChanged: Event<void>;
    private readonly _api;
    private _startForwardQueue;
    private _forwardings;
    get forwardings(): Forwarding[];
    /**@internal */
    constructor(args: {
        channel: SkyWayChannelImpl;
        id: string;
        name?: string;
        metadata?: string | undefined;
        plugin: SfuClientPlugin;
        api: SfuRestApiClient;
        context: SkyWayContext;
        transportRepository: TransportRepository;
    });
    /**@private */
    _getConnection(localPersonId: string): SFUConnection;
    /**@private */
    _getOrCreateConnection(localPerson: LocalPersonImpl): SFUConnection;
    /**@private */
    private _createConnection;
    /**
     * @description {japanese} StreamのPublicationをForwardingする
     */
    startForwarding(publication: Publication<LocalVideoStream | LocalAudioStream>, configure?: ForwardingConfigure): Promise<Forwarding>;
    private _startForwarding;
    private listenStopForwardEvent;
    /**
     * @description {japanese} Forwardingを停止する
     */
    stopForwarding: (target: string | Forwarding) => Promise<void>;
    /**
     * @description {japanese} MemberがSfuBotかどうか識別する
     */
    static IsSfuBotMember: (member: Member) => member is SfuBotMember;
}
//# sourceMappingURL=member.d.ts.map