"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SfuClientPlugin = void 0;
const common_1 = require("@skyway-sdk/common");
const core_1 = require("@skyway-sdk/core");
const sfu_api_client_1 = require("@skyway-sdk/sfu-api-client");
const transportRepository_1 = require("./connection/transport/transportRepository");
const const_1 = require("./const");
const member_1 = require("./member");
const log = new common_1.Logger('packages/sfu-client/src/plugin.ts');
class SfuClientPlugin extends core_1.SkyWayPlugin {
    constructor(_options = {}) {
        super();
        this.subtype = member_1.SfuBotMember.subtype;
        /**@private */
        this._createRemoteMember = (channel, sfuBot) => {
            const member = new member_1.SfuBotMember(Object.assign(Object.assign({}, this._context), { channel, id: sfuBot.id, name: sfuBot.name, metadata: sfuBot.metadata, plugin: this, api: this._api, context: this._context, transportRepository: this._transportRepository }));
            return member;
        };
        /**
         * @description {japanese} SFU BotをChannelに呼び出す
         */
        this.createBot = (channel) => __awaiter(this, void 0, void 0, function* () {
            var _a;
            const botId = yield this._api.createBot({
                appId: this._context.authToken.scope.app.id,
                channelId: channel.id,
            });
            const member = (_a = channel._getMember(botId)) !== null && _a !== void 0 ? _a : (yield channel.onMemberJoined
                .watch((e) => e.member.id === botId, this._context.config.rtcApi.timeout)
                .catch((error) => {
                throw new common_1.SkyWayError({
                    type: 'timeout',
                    message: 'SfuClientPlugin onMemberJoined',
                    payload: { error },
                    path: log.prefix,
                });
            })).member;
            return member;
        });
        /**
         * @description {japanese} SFU BotをChannelから削除する
         */
        this.deleteBot = (channel, botId) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((r, f) => {
                this._api.deleteBot({ botId }).catch(f);
                channel.onMemberLeft
                    .watch((e) => e.member.id === botId, this._context.config.rtcApi.timeout)
                    .catch(() => {
                    f(new common_1.SkyWayError({
                        type: 'timeout',
                        message: 'onMemberLeft',
                        path: log.prefix,
                    }));
                })
                    .then(() => r());
            });
        });
        this.options = Object.assign(Object.assign({}, const_1.defaultSfuClientPluginOptions), _options);
        this._onContextAttached.once((context) => {
            this._api = new sfu_api_client_1.SfuRestApiClient(context.authTokenString, Object.assign(Object.assign({}, this.options), { log: context.config.log }));
            this._transportRepository = new transportRepository_1.TransportRepository(context, this._api);
            context._onTokenUpdated.add((token) => {
                this._api.updateToken(token);
            });
        });
        this._whenDisposeLocalPerson = (person) => __awaiter(this, void 0, void 0, function* () {
            this._transportRepository.deleteTransports(person.id);
        });
    }
}
exports.SfuClientPlugin = SfuClientPlugin;
//# sourceMappingURL=plugin.js.map